/**
 * 
 */
package de.ixxat.vci3;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.bal.*;

/**
 * This class implements the interface {@link IVciDevice} which represents a VCI
 * device object. When no longer needed the VCI device object has to be disposed
 * using the {@link IDisposable} interface. Get a VCI device object via the
 * {@link IVciDeviceManager#OpenDevice(long)}.
 * 
 * @author Michael Ummenhofer
 * 
 */
class VciDevice extends CInterface implements IVciDevice 
{
  private static final int GetDeviceInfo  = 3;
  private static final int GetDeviceCaps  = 4;
  private static final int OpenComponent  = 5;

  // Interface functions
  private CFunction m_fpGetDeviceInfo   = null;
  private CFunction m_fpGetDeviceCaps   = null;
  private CFunction m_fpOpenComponent   = null;

  // Interface Pointer
  private CPointer  m_IVciDevice   = null; //Interface Pointer itself (pThis)

  /** VCI Device Capabilities */
  private VciDeviceCapabilities m_oVciDeviceCaps  = new VciDeviceCapabilities();

  /** VCI Device Information */
  private VciDeviceInfo m_oVciDeviceInfo          = new VciDeviceInfo();

  /**
   * constructor
   * 
   * @param pIVciDevice
   * @throws Throwable
   */
  protected VciDevice(CMalloc pIVciDevice) throws Throwable
  {
    super(pIVciDevice);
    
    try
    {
      // Retrieve Interface Pointer Address
      m_IVciDevice = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpGetDeviceInfo   = new CFunction(m_IVciDevice, GetDeviceInfo,  "JNI");
      m_fpGetDeviceCaps   = new CFunction(m_IVciDevice, GetDeviceCaps,  "JNI");
      m_fpOpenComponent   = new CFunction(m_IVciDevice, OpenComponent,  "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_IVciDevice        = null;
      m_fpGetDeviceInfo   = null;
      m_fpGetDeviceCaps   = null;
      m_fpOpenComponent   = null;
      
      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_IVciDevice        = null;
      m_fpGetDeviceInfo   = null;
      m_fpGetDeviceCaps   = null;
      m_fpOpenComponent   = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * @see de.ixxat.vci3.IVciDevice#GetDeviceInfo
   * @throws Throwable
   */
  public VciDeviceInfo GetDeviceInfo() throws Throwable
  {
    int hResult = 0;

    // Create VCIDEVICEINFO Memory
    CMalloc pVciDeviceInfo = new CMalloc(304);
    try
    {
      hResult = m_fpGetDeviceInfo.callInt(new Object[] { m_IVciDevice, pVciDeviceInfo });
      if (hResult != 0)
      {
        throw new VciException("IVciDevice->GetDeviceInfo failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oVciDeviceInfo.fromCPointer(pVciDeviceInfo);
      }
    } 
    finally
    {
      pVciDeviceInfo.free();
    }
    
    return m_oVciDeviceInfo;
  }
  
  /**
   * @see de.ixxat.vci3.IVciDevice#GetDeviceCaps()
   * @throws Throwable
   */
  public VciDeviceCapabilities GetDeviceCaps() throws Throwable
  {
    int hResult = 0;

    // Create VCIDEVICECAPS Memory
    CMalloc pVciDeviceCaps = new CMalloc(2 + 2 * VciDeviceCapabilities.VCI_MAX_BUSCTRL);
    try
    {
      hResult = m_fpGetDeviceCaps.callInt(new Object[] { m_IVciDevice, pVciDeviceCaps });
      if (hResult != 0)
      {
        throw new VciException("IVciDevice->GetDeviceCaps failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oVciDeviceCaps.fromCPointer(pVciDeviceCaps);
      }
    } 
    finally
    {
      pVciDeviceCaps.free();
    }
    
    return m_oVciDeviceCaps;
  }
  
  /**
   * @see de.ixxat.vci3.IVciDevice#OpenBusAccessLayer()
   * @throws Throwable
   */
  public IBalObject OpenBusAccessLayer() throws Throwable
  {
    BalObject oBalObject = null;
    int       hResult    = 0;

    // Create BalObject Interface Memory
    CMalloc pIBalObject = new CMalloc(8);
    CGuid pClassId      = new CGuid(0x3CB880F4,0x2791,0x40BD,new short[]{0xB8,0x52,0x09,0x34,0x9D,0x1A,0x7B,0x37}); //CLSID_VCIBAL
    CGuid pInterfaceId  = new CGuid(0xA2B958AA,0x9188,0x4CDB,new short[]{0x83,0x07,0x25,0xD9,0xF1,0xF8,0x43,0xE9}); //IID_IBalObject
    try
    {
      pIBalObject.setLong(0, 0);

      // Retrieve IVciDeviceManger Interface
      hResult = m_fpOpenComponent.callInt(new Object[]{ m_IVciDevice, pClassId, pInterfaceId, pIBalObject});
      if(hResult != 0)
      {
        throw new VciException("IVciDevice->OpenComponent failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // Create new BAL Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the BAL Object
        oBalObject = new BalObject(pIBalObject);
      }
    }
    catch(Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pIBalObject.free();
      throw oException;
    }
    finally
    {
      // Free GUID Memory
      pClassId.free();
      pClassId = null;
      
      pInterfaceId.free();
      pInterfaceId = null;;
    }
    
    return oBalObject;
  }

}
