/**
 * 
 */
package de.ixxat.vci3.bal;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;
import de.ixxat.vci3.bal.can.*;
import de.ixxat.vci3.bal.lin.*;

/**
 * This class implements the interface {@link IBalObject} which represents a BAL
 * (Bus Access Layer) object. When no longer needed the BAL object has to be
 * disposed using the {@link IDisposable} interface. Get the BAL object of a
 * device by using method {@link de.ixxat.vci3.IVciDevice#OpenBusAccessLayer()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class BalObject extends CInterface implements IBalObject
{
  private static final int GetFeatures    = 3;
  private static final int OpenSocket     = 4;

  // Interface functions
  private CFunction m_fpGetFeatures     = null;
  private CFunction m_fpOpenSocket      = null;

  // Interface Pointer
  private CPointer  m_IBalObject   = null; //Interface Pointer itself (pThis)

  /** BAL Features */
  private BalFeatures m_oBalFeatures      = new BalFeatures();

  //TODO: Is there an other way to create this object -> "protected"
  /**
   * Constructor
   *  
   * @param pIBalObject
   * @throws Throwable
   */
  public BalObject(CMalloc pIBalObject) throws Throwable
  {
    super(pIBalObject);
    
    try
    {
      // Retrieve Interface Pointer Address
      m_IBalObject = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpGetFeatures     = new CFunction(m_IBalObject, GetFeatures,    "JNI");
      m_fpOpenSocket      = new CFunction(m_IBalObject, OpenSocket,     "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_IBalObject        = null;
      m_fpGetFeatures     = null;
      m_fpOpenSocket      = null;
      
      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_IBalObject        = null;
      m_fpGetFeatures     = null;
      m_fpOpenSocket      = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * @see IBalObject#GetFeatures()
   * @throws Throwable
   */
  public BalFeatures GetFeatures() throws Throwable
  {
    int hResult = 0;

    // Create BALFEATURES Memory
    CMalloc pBalFeatures = new CMalloc(6 + 2 * BalFeatures.BAL_MAX_SOCKETS);
    try
    {
      hResult = m_fpGetFeatures.callInt(new Object[] { m_IBalObject, pBalFeatures });
      if (hResult != 0)
      {
        throw new VciException("IBalResource->GetFeatures failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oBalFeatures.fromCPointer(pBalFeatures);
      }
    } 
    finally
    {
      pBalFeatures.free();
    }
    
    return m_oBalFeatures;
  }

  /**
   * @see IBalObject#OpenSocket(int, int)
   */
  public IBalResource OpenSocket(int dwPortNumber, int dwSocketType) throws Throwable
  {
    IBalResource oBalResource = null;
    int          hResult    = 0;

    // Create BalObject Interface Memory
    CMalloc pIBalResource = new CMalloc(8);
    CGuid   pInterfaceId  = null;
    
    // Which socket Type?
    switch(dwSocketType)
    {
      case IBalResource.IID_ICanControl:
      {
        pInterfaceId = new CGuid(0xD213C143,0x7039,0x4D1D,new short[]{0xAC,0xED,0x13,0x90,0x28,0x9C,0x8E,0xBA});
        break;
      }
      case IBalResource.IID_ICanScheduler:
      {
        pInterfaceId = new CGuid(0xC936498F,0xEC52,0x4C74,new short[]{0xB3,0xFE,0xA4,0x72,0xBA,0x73,0x65,0xBA});
        break;
      }
      case IBalResource.IID_ICanSocket:
      {
        pInterfaceId = new CGuid(0x48965C25,0x9F8D,0x4E4D,new short[]{0x8D,0xE6,0xF0,0x29,0x80,0x8C,0x9E,0xD2});
        break;
      }

      case IBalResource.IID_ILinControl:
      {
        pInterfaceId = new CGuid(0x4D58EF55,0x4F01,0x451E,new short[]{0x91,0x1F,0x17,0xC2,0x23,0xA8,0xB8,0xF0});
        break;
      }
      case IBalResource.IID_ILinSocket:
      {
        pInterfaceId = new CGuid(0xEF865638,0x6D07,0x4A46,new short[]{0xA8,0x38,0x4B,0xAF,0xED,0x35,0x9B,0x1A});
        break;
      }
      default:
      {
        throw new IllegalArgumentException("dwSocketType out of range");
      }
    }
    
    try
    {
      pIBalResource.setLong(0, 0);

      // Retrieve BalResource Interface
      hResult = m_fpOpenSocket.callInt(new Object[]{ m_IBalObject, new Integer(dwPortNumber), pInterfaceId, pIBalResource});
      if(hResult != 0)
      {
        throw new VciException("IBalResource->OpenSocket failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
      else
      {
        // Create new BalResource Object and set Interface Pointer
        // after a successful creation of the Object the allocated memory
        // is in the responsibility of the BalResource Object
        // Which socket Type?
        switch(dwSocketType)
        {
          case IBalResource.IID_ICanControl:
          {
            oBalResource = new CanControl(pIBalResource);
            break;
          }
          case IBalResource.IID_ICanScheduler:
          {
            oBalResource = new CanScheduler(pIBalResource);
            break;
          }
          case IBalResource.IID_ICanSocket:
          {
            oBalResource = new CanSocket(pIBalResource);
            break;
          }

          case IBalResource.IID_ILinControl:
          {
            oBalResource = new LinControl(pIBalResource);
            break;
          }
          case IBalResource.IID_ILinSocket:
          {
            oBalResource = new LinSocket(pIBalResource);
            break;
          }
        }//end switch
      }// end else
    }
    catch(Exception oException)
    {
      // On error free allocated Memory and pass error to next handler
      pIBalResource.free();
      throw oException;
    }
    finally
    {
      // Free GUID Memory
      pInterfaceId.free();
    }

    return oBalResource;
  }
 
 
}
