/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.vci3.IDisposable;
import de.ixxat.vci3.bal.IBalResource;

/**
 * This interface represents a CAN control unit and is used to control a CAN
 * line. When no longer needed the CAN communication channel object has to be
 * disposed using the {@link IDisposable} interface. Controlling consists of
 * initialization, starting/stopping the CAN line and adjusting filter settings.
 * A CAN control object can be obtained via method
 * {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)}. The CAN control
 * cannot be opened twice at the same time. Therefore a second try to open the
 * CAN control via {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)}
 * fails until the successfully opened CAN control object is discarded.
 * 
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface ICanControl extends IBalResource
{
  /** reception of 11-bit id messages */
  public static final int CAN_OPMODE_STANDARD = 0x01;
  /** reception of 29-bit id messages */
  public static final int CAN_OPMODE_EXTENDED = 0x02;
  /** enable reception of error frames */
  public static final int CAN_OPMODE_ERRFRAME = 0x04;
  /** listen only mode (TX passive) */
  public static final int CAN_OPMODE_LISTONLY = 0x08;
  /** use low speed bus interface */
  public static final int CAN_OPMODE_LOWSPEED = 0x10;

  /** select standard filter (11-bit) */
  public static final int CAN_FILTER_STD = 1;
  /** select extended filter (29-bit) */
  public static final int CAN_FILTER_EXT = 2;

  /**
   * This method detects the actual bit rate of the CAN line to which the
   * controller is connected.
   * <p>
   * <strong>Remarks:</strong><blockquote> The method detects the actual bit
   * rate beginning at the first entry within the specified array and switches
   * to the next entry until the correct baud rate is detected or the table
   * limit is reached. If the time between two successive receive messages
   * exceed the value specified by the timeout parameter, the method throws a
   * exception. The total execution time of the method can be
   * determined by the following formula: <code>TotalExecutionTime [ms] = timeout *
   * bitrateTable.Length </code>
   * </blockquote>
   * 
   * @param dwTimeout
   *          Timeout in milliseconds to wait between two successive receive
   *          messages.
   * @param aoBitrateTable
   *          One-dimensional array of initialized CanBitrate objects which
   *          contains possible values for the bit timing register to be tested.
   * @return {@link CanBitrate} If the method succeeds it returns the detected
   *         CanBitrate.
   * @throws Throwable
   */
  public CanBitrate DetectBaud(int dwTimeout, CanBitrate[] aoBitrateTable) throws Throwable;
  
  /**
   * This method initialize the CAN line in the specified operating mode and bit
   * transfer rate. The method also performs a reset of the CAN controller
   * hardware and disables the reception of CAN messages.
   * <p>
   * <strong>Remarks:</strong><blockquote> The operatingMode parameter defines
   * the operating mode of the CAN controller. The operating mode can be a
   * combination of the following primary operating mode flags: <blockquote>
   * <li> {@link #CAN_OPMODE_STANDARD} - standard frame format (11 bit
   * identifier)</li>
   * <li>{@link #CAN_OPMODE_EXTENDED} - extended frame format (29 bit
   * identifier)</li>
   * </blockquote> Optionally, the following flags can be combined with the
   * primary operating mode flags: <blockquote>
   * <li>{@link #CAN_OPMODE_ERRFRAME} - accept error frames</li>
   * <li>{@link #CAN_OPMODE_LISTONLY} - listen only mode</li>
   * <li>{@link #CAN_OPMODE_LOWSPEED} - use low speed bus interface</li>
   * </blockquote> The bit transfer rate for the CAN controller is specified by
   * the bitrate parameter. The timing value must be set according to the values
   * of the bit timing register 0 and 1 of an Philips SJA1000 CAN controller at
   * a frequency of 16 MHz. See the Philips SJA1000 datasheet for more
   * information of how to compute the timing values for a given bit rate.
   * </blockquote>
   * 
   * @param dwOperatingMode
   *          Operating mode of the CAN controller
   * @param oCanBitrate
   *          Bit timing value according to Philips SJA1000 at 16MHz.
   * @throws Throwable
   */
  public void InitLine(int dwOperatingMode, CanBitrate oCanBitrate) throws Throwable;

  /**
   * This method reset the CAN line to it's initial state. The method aborts a
   * currently busy transmit message and switch the CAN controller into init
   * mode. The method additionally clears the standard and extended mode ID
   * filter.
   * 
   * @throws Throwable
   */
  public void ResetLine() throws Throwable;

  /**
   * This method starts the CAN line and switch it into running mode. After
   * starting the CAN line, CAN messages can be transmitted over the message
   * channel.
   * 
   * @throws Throwable
   */
  public void StartLine() throws Throwable;

  /**
   * This method stops the CAN line an switch it into init mode. After stopping
   * the CAN controller no further CAN messages are transmitted over the message
   * channel. Other than ResetLine, this method does not abort a currently busy
   * transmit message and does not clear the standard and extended mode ID
   * filter.
   * 
   * @throws Throwable
   */
  public void StopLine() throws Throwable;

  /**
   * Gets the current status of the CAN Line.
   * 
   * @return {@link CanLineStatus} CanLineStatus is used to signal the status of
   *         a CAN controller.
   * @throws Throwable
   */
  public CanLineStatus GetLineStatus() throws Throwable;

  /**
   * This method sets the global acceptance filter. The global acceptance filter
   * enables the reception of CAN message identifiers specified by the bit
   * patterns passed in dwCode and dwMask. The message IDs enabled by this
   * method are always accepted, even if the specified IDs are not registered
   * within the filter list (see also {@link #AddFilterIds}).
   * <p>
   * <strong>Remarks:</strong><blockquote> The acceptance filter is defined by
   * the acceptance code and acceptance mask. The bit pattern of CANIDs to be
   * received are defined by the acceptance code. The corresponding acceptance
   * mask allow to define certain bit positions to be don't care (bit x = 0).
   * </blockquote>
   * <p>
   * <strong>Example:</strong><blockquote> The values in code and mask have
   * the following format: <tt><br><br>
   * dwSelect = {@link #CAN_FILTER_STD}<br>
   * <br>
   *  &nbsp&nbsp&nbsp&nbsp+----+----+----+----+ ~ +----+----+ ~ +---+---+---+---+<br>
   *  bit | 31 | 30 | 29 | 28 |&nbsp &nbsp| 13 | 12 |&nbsp &nbsp| 3 | 2 | 1 | 0 |<br>
   *  &nbsp&nbsp&nbsp&nbsp+----+----+----+----+ ~ +----+----+ ~ +---+---+---+---+<br>
   *  &nbsp&nbsp&nbsp&nbsp|&nbsp 0 |&nbsp 0 |&nbsp 0 |&nbsp 0 |&nbsp &nbsp|&nbsp 0 |ID11|&nbsp &nbsp|ID2|ID1|ID0|RTR|<br>
   *  &nbsp&nbsp&nbsp&nbsp+----+----+----+----+ ~ +----+----+ ~ +---+---+---+---+<br>
   *<br>
   * dwSelect = {@link #CAN_FILTER_EXT}<br>
   * <br>
   *  &nbsp&nbsp&nbsp&nbsp+----+----+----+----+ ~ +----+----+ ~ +---+---+---+---+<br>
   *  bit | 31 | 30 | 29 | 28 |&nbsp &nbsp| 13 | 12 |&nbsp &nbsp| 3 | 2 | 1 | 0 |<br>
   *  &nbsp&nbsp&nbsp&nbsp+----+----+----+----+ ~ +----+----+ ~ +---+---+---+---+<br>
   *  &nbsp&nbsp&nbsp&nbsp|&nbsp 0 |&nbsp 0 |ID28|ID27|&nbsp &nbsp|ID12|ID11|&nbsp &nbsp|ID2|ID1|ID0|RTR|<br>
   *  &nbsp&nbsp&nbsp&nbsp+----+----+----+----+ ~ +----+----+ ~ +---+---+---+---+<br>
   *      <br>
   * </tt> The following example demonstrates how to compute the code and mask
   * values to enable the standard IDs in the range from 0x100 to 0x103 whereas
   * RTR is 0. <tt><br><br>
   * code &nbsp&nbsp= 001 0001 1000 0<br>
   * mask &nbsp&nbsp= 111 1111 1100 1<br>
   * result = 001 0001 10xx 0<br>
   *<br>
   * enabled IDs:<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 001 0000 0000 0 (0x100, RTR = 0)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 001 0000 0001 0 (0x101, RTR = 0)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 001 0000 0010 0 (0x102, RTR = 0)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 001 0000 0011 0 (0x103, RTR = 0)<br>
   * </blockquote></tt>
   * 
   * @param dwSelect
   *          Filter selection. This parameter can be either
   *          {@link #CAN_FILTER_STD} to select the 11-bit acceptance filter, or
   *          {@link #CAN_FILTER_STD} to select the 29-bit acceptance filter.
   * @param dwCode
   *          Acceptance code inclusive RTR bit.
   * @param dwMask
   *          Mask that specifies the relevant bits within code. Relevant bits
   *          are specified by a 1 in the corresponding bit position, non
   *          relevant bits are 0.
   * @throws Throwable
   */
  public void SetAccFilter(int dwSelect, int dwCode, int dwMask) throws Throwable;

  /**
   * This method registers the specified CAN message identifier or group of
   * identifiers at the specified filter list. IDs registered within the filter
   * list are accepted for reception. The method can only be called if the CAN
   * controller is in 'init' mode.
   * <p>
   * <strong>Example:</strong><blockquote> </tt> The following example
   * demonstrates how to compute the code and mask values to register the
   * standard IDs in the range from 0x518 to 0x51B whereas RTR is 1. <tt><br>
   * <br>
   * code &nbsp&nbsp= 0101 0001 1000 1<br>
   * mask &nbsp&nbsp= 0111 1111 1100 1<br>
   * result = 0101 0001 10xx 1<br>
   * <br>
   * IDs registered by this method:<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1000 1 (0x518, RTR = 1)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1001 1 (0x519, RTR = 1)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1010 1 (0x51A, RTR = 1)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1011 1 (0x51B, RTR = 1)<br>
   * </blockquote></tt>
   * 
   * @param dwSelect
   *          Filter selection. This parameter can be either
   *          {@link #CAN_FILTER_STD} to select the 11-bit acceptance filter, or
   *          {@link #CAN_FILTER_STD} to select the 29-bit acceptance filter.
   * @param dwCode
   *          Message identifier (inclusive RTR) to add to the filter list.
   * @param dwMask
   *          Mask that specifies the relevant bits within code. Relevant bits
   *          are specified by a 1 in the corresponding bit position, non
   *          relevant bits are 0.
   * @throws Throwable
   */
  public void AddFilterIds(int dwSelect, int dwCode, int dwMask) throws Throwable;

  /**
   * This method registers the specified CAN message identifier or group of
   * identifiers at the specified filter list. IDs registered within the filter
   * list are accepted for reception. The method can only be called if the CAN
   * controller is in 'init' mode.
   * <p>
   * <strong>Example:</strong><blockquote> </tt> The following example
   * demonstrates how to compute the code and mask values to register the
   * standard IDs in the range from 0x518 to 0x51B whereas RTR is 1. <tt><br>
   * <br>
   * code &nbsp&nbsp= 0101 0001 1000 1<br>
   * mask &nbsp&nbsp= 0111 1111 1100 1<br>
   * result = 0101 0001 10xx 1<br>
   * <br>
   * IDs removed by this method:<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1000 1 (0x518, RTR = 1)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1001 1 (0x519, RTR = 1)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1010 1 (0x51A, RTR = 1)<br>
   * &nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp 0101 0001 1011 1 (0x51B, RTR = 1)<br>
   * </blockquote></tt>
   * 
   * @param dwSelect
   *          Filter selection. This parameter can be either
   *          {@link #CAN_FILTER_STD} to select the 11-bit acceptance filter, or
   *          {@link #CAN_FILTER_STD} to select the 29-bit acceptance filter.
   * @param dwCode
   *          Message identifier (inclusive RTR) to remove from the filter list.
   * @param dwMask
   *          Mask that specifies the relevant bits within code. Relevant bits
   *          are specified by a 1 in the corresponding bit position, non
   *          relevant bits are 0.
   * @throws Throwable
   */
  public void RemFilterIds(int dwSelect, int dwCode, int dwMask) throws Throwable;
}
