/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.vci3.IDisposable;
import de.ixxat.vci3.bal.IBalResource;

/**
 * This interface represents a CAN scheduler. A CAN scheduler provides the
 * functionality to cyclically transmit CAN messages. Optionally such a transmit
 * message can automatically be altered from one transmission to the next
 * (identifier or a piece of data). The CAN scheduler provides methods to
 * establish, start and stop cyclic transmit messages for a CAN line. When no
 * longer needed the CAN scheduler object has to be disposed using the
 * {@link IDisposable} interface. A CAN scheduler object can be obtained via
 * method {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)}. The CAN
 * scheduler cannot be opened twice at the same time. Therefore a second try to
 * open the CAN scheduler via
 * {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)} fails until the
 * successfully opened CAN scheduler object is explicitly disposed.
 * 
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface ICanScheduler extends IBalResource
{
  /** Maximum possible cyclic transmit messages. */
  public static final int CAN_MAX_CTX_MSGS = 16;

  /**
   * Definition for endless transmission of CAN Messages. Can be used with
   * method {@link #StartMessage(int, int)}
   */
  public static final int CAN_MSG_SCHEDULE_ENDLESS = 0;

  /**
   * This method resumes execution of the scheduler and starts processing of all
   * currently registered message.
   * 
   * @throws Throwable
   */
  public void Resume() throws Throwable;

  /**
   * This method suspends execution of the scheduler and stops processing of all
   * currently registered messages.
   * 
   * 
   * @throws Throwable
   */
  public void Suspend() throws Throwable;

  /**
   * This method suspends execution of the scheduler and removes all currently
   * registered messages.
   * 
   * 
   * @throws Throwable
   */
  public void Reset() throws Throwable;

  /**
   * This method retrieves the current status of the scheduler and all
   * registered messages.
   * 
   * @return {@link CanSchedulerStatus}
   * @throws Throwable
   */
  public CanSchedulerStatus GetStatus() throws Throwable;

  /**
   * This method adds a new cyclic transmit message to the scheduler.
   * 
   * @param oCanCyclicTxMessage
   *          Reference to the initialized cyclic transmit message
   *          {@link CanCyclicTxMessage} to add.
   * @param dwIndex
   *          Index of message between 0 and {@link #CAN_MAX_CTX_MSGS}
   * @throws Throwable
   */
  public void AddMessage(CanCyclicTxMessage oCanCyclicTxMessage, int dwIndex) throws Throwable;

  /**
   * This method removes the specified cyclic transmit message from the
   * scheduler.
   * 
   * 
   * @throws Throwable
   */
  public void RemMessage(int dwIndex) throws Throwable;

  /**
   * This method starts processing of the specified cyclic transmit message.
   * 
   * @param dwIndex
   *          Index of message between 0 and {@link #CAN_MAX_CTX_MSGS}
   * @param dwCount
   *          Number of repetitions the message should be sent. If this
   *          parameter is set to {@link #CAN_MSG_SCHEDULE_ENDLESS}, the
   *          message is sent endlessly.
   * @throws Throwable
   */
  public void StartMessage(int dwIndex, int dwCount) throws Throwable;

  /**
   * This method stops processing of this cyclic transmit message.
   * 
   * @param dwIndex
   *          Index of message between 0 and {@link #CAN_MAX_CTX_MSGS}
   * @throws Throwable
   */
  public void StopMessage(int dwIndex) throws Throwable;

}
