/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.ZeroHex;
import de.ixxat.jni.*;
import de.ixxat.vci3.*;

/**
 * This class implements the interface {@link ILinControl} which represents a LIN
 * control unit and is used to control a LIN line.
 * 
 * @see ILinControl
 * @author Michael Ummenhofer
 * 
 */
public class LinControl extends CInterface implements ILinControl
{

  private static final int InitLine       = 3;
  private static final int ResetLine      = 4;
  private static final int StartLine      = 5;
  private static final int StopLine       = 6;
  private static final int WriteMessage   = 7;
  private static final int GetLineStatus  = 8;

  // Interface functions
  private CFunction m_fpInitLine        = null;
  private CFunction m_fpResetLine       = null;
  private CFunction m_fpStartLine       = null;
  private CFunction m_fpStopLine        = null;
  private CFunction m_fpWriteMessage    = null;
  private CFunction m_fpGetLineStatus   = null;

  // Interface Pointer
  private CPointer  m_ILinControl   = null; //Interface Pointer itself (pThis)

  /** LIN Line Status */
  private LinLineStatus m_oLinLineStatus = new LinLineStatus();

  //TODO: Is there an other way to create this object -> "protected"
  /**
   * Constructor
   * 
   * @param pILinControl
   * @throws Throwable
   */
  public LinControl(CMalloc pILinControl) throws Throwable
  {
    super(pILinControl);
    
    try
    {
      // Retrieve Interface Pointer Address
      m_ILinControl = super.m_Interface;
      
      // Retrieve Interface Methods
      m_fpInitLine        = new CFunction(m_ILinControl, InitLine,       "JNI");
      m_fpResetLine       = new CFunction(m_ILinControl, ResetLine,      "JNI");
      m_fpStartLine       = new CFunction(m_ILinControl, StartLine,      "JNI");
      m_fpStopLine        = new CFunction(m_ILinControl, StopLine,       "JNI");
      m_fpWriteMessage    = new CFunction(m_ILinControl, WriteMessage,   "JNI");
      m_fpGetLineStatus   = new CFunction(m_ILinControl, GetLineStatus,  "JNI");
    }
    catch(Exception oException)
    {
      // Release all Pointers without freeing memory 
      m_ILinControl        = null;
      m_fpInitLine         = null;
      m_fpResetLine        = null;
      m_fpStartLine        = null;
      m_fpStopLine         = null;
      m_fpWriteMessage     = null;
      m_fpGetLineStatus    = null;
      
      throw oException;
    }
  }

  /**
   * Clean up Object
   */
  protected void finalize() throws Throwable
  {
    try
    {
      Dispose();
    }
    finally
    {
      // Call finalize of base class
      super.finalize();
    }
  }
  
  /**
   * @see de.ixxat.jni.CInterface#Dispose()
   * @throws Throwable
   */
  public void Dispose() throws Throwable
  {
    try
    {
      m_ILinControl        = null;
      m_fpInitLine         = null;
      m_fpResetLine        = null;
      m_fpStartLine        = null;
      m_fpStopLine         = null;
      m_fpWriteMessage     = null;
      m_fpGetLineStatus    = null;
    }
    finally
    {
      // Call Base Class
      super.Dispose();
    }
  }
  
  /**
   * @see ILinControl#InitLine(byte, short)
   * @throws Throwable
   */
  public void InitLine(byte bOperatingMode, short wLinBitrate) throws Throwable
  {
    int hResult = 0;

    // Create LIN InitLine structure Memory
    CMalloc pLinInitLine = new CMalloc(4);

    try
    {
      // Initialize LININITLINE structure
      pLinInitLine.setByte(0, bOperatingMode);
      pLinInitLine.setByte(1, (byte)0); // bReserved
      pLinInitLine.setShort(2, wLinBitrate);

      // Initialize LIN Controller
      hResult = m_fpInitLine.callInt(new Object[]{ m_ILinControl, pLinInitLine});
      if(hResult != 0)
      {
        throw new VciException("ILinControl->InitLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
      // Free Structure Memory
      pLinInitLine.free();
    }
    
  }
  
  /**
   * @see ILinControl#ResetLine()
   * @throws Throwable
   */
  public void ResetLine() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpResetLine.callInt(new Object[]{ m_ILinControl });
      if(hResult != 0)
      {
        throw new VciException("ILinControl->ResetLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
  
  /**
   * @see ILinControl#StartLine()
   * @throws Throwable
   */
  public void StartLine() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpStartLine.callInt(new Object[]{ m_ILinControl });
      if(hResult != 0)
      {
        throw new VciException("ILinControl->StartLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }
  
  /**
   * @see ILinControl#StopLine()
   * @throws Throwable
   */
  public void StopLine() throws Throwable
  {
    int hResult = 0;

    try
    {
      hResult = m_fpStopLine.callInt(new Object[]{ m_ILinControl });
      if(hResult != 0)
      {
        throw new VciException("ILinControl->StopLine failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      }
    }
    finally
    {
    }
  }

  /**
   * @see ILinControl#WriteMessage(boolean, LinMessage)
   * @throws Throwable
   */
  public void WriteMessage(boolean fSend, LinMessage oLinMsg) throws Throwable
  {
    int hResult = 0;

    // Create LINMSG Memory
    CMalloc pLinMsg = new CMalloc(16);
    try
    {
      // Default: Put Message in LIN Response table
      int nSend = 0;
      
      // Send Message directly on the LIN Bus
      if(fSend)
        nSend = 1;

      // Update LINMSG structure
      oLinMsg.toCPointer(pLinMsg);
      
      hResult = m_fpWriteMessage.callInt(new Object[] { m_ILinControl, new Integer(nSend), pLinMsg});
      if (hResult != 0)
      {
        throw new VciException("ILinControl->WriteMessage failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
    } 
    finally
    {
      pLinMsg.free();
    }
    
  }

  /**
   * @see ILinControl#GetLineStatus()
   * @throws Throwable
   */
  public LinLineStatus GetLineStatus() throws Throwable
  {
    int hResult = 0;

    // Create LINLINESTATUS Memory
    CMalloc pLinLineStatus = new CMalloc(8);
    try
    {
      hResult = m_fpGetLineStatus.callInt(new Object[] { m_ILinControl, pLinLineStatus });
      if (hResult != 0)
      {
        throw new VciException("ILinControl->GetLineStatus failed with errorcode 0x" + ZeroHex.toZeroHexString(hResult), hResult);
      } 
      else
      {
        m_oLinLineStatus.fromCPointer(pLinLineStatus);
      }
    } 
    finally
    {
      pLinLineStatus.free();
    }
    
    return m_oLinLineStatus;
  }
  
  
}
